; Copyright 2019-2021 Sean Porter (Glitchfinder)
;
; Permission is hereby granted, free of charge, to any person obtaining a copy
; of this software and associated documentation files (the "Software"), to deal
; in the Software without restriction, including without limitation the rights
; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
; copies of the Software, and to permit persons to whom the Software is
; furnished to do so, subject to the following conditions:
;
; The above copyright notice and this permission notice shall be included in all
; copies or substantial portions of the Software.
;
; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
; SOFTWARE.
;------------------------------------------------------------------------------/

Scriptname zYAWA_Settlement_BOS Extends ObjectReference

zYAWA_Patrol_BOS[]  Property Patrols    Auto Hidden ; This settlement's patrols
ObjectReference Property Center     Auto Hidden ; The patrol endoint
ObjectReference Property Entrance   Auto Hidden ; The 'entrance' used by patrols
WorkshopScript  Property Settlement Auto Hidden ; This settlement's workshop
zYAWA_Region_BOS    Property Region     Auto Hidden ; This settlement's region

;------------------------------------------------------------------------------\
; Initialize this settlement
;------------------------------------------------------------------------------/
Function Initialize(zYAWA_Manager_BOS m, WorkshopScript w, Int id)
	; Set this patrol's settlement and initialize its patrol array
	Settlement = w
	Patrols = New zYAWA_Patrol_BOS[128]

	; Get the object spawning information
	ObjectReference d = m.DataPoint
	Form p = m.Patrol

	; Get the initial center and entrance points
	Center = m.BSettlements[id].GetLinkedRef(m.Center)
	Entrance = Center

	; Check to see if the door should be moved
	Int ps = CheckDoor(m, w)

	; If the door was moved
	If (ps >= 0)
		; Delete the move object and remove it from the list
		(m.Doors.GetAt(ps) As ObjectReference).Delete()
		m.Doors.RemoveAddedForm(m.Doors.GetAt(ps))
	EndIf

	; Get this settlement's region
	Region = m.Regions[m.SortSettlement(Entrance)] As zYAWA_Region_BOS

	; Start with the first patrol
	Int i = -1

	; Go through all potential patrols for this settlement
	While i < 127
		; Increment the patrol id
		i += 1

		; Get this patrol's settlement
		zYAWA_Settlement_BOS s = m.Settlements[i]

		; If the other settlement exists and is not this settlement
		If (i != id && s != None && i < id)
			; Create a patrol object and assign it to both settlements
			Patrols[i] = d.PlaceAtMe(p) As zYAWA_Patrol_BOS
			s.Patrols[id] = Patrols[i]

			; Initialize the patrol object
			Patrols[i].Initialize(id, i, m)
		EndIf
	EndWhile
EndFunction

;------------------------------------------------------------------------------\
; Update this settlement's patrol settings
;------------------------------------------------------------------------------/
Function Settings(zYAWA_Manager_BOS m, zYAWA_Region_BOS r)
	; If this settlement doesn't belong in the given region
	If (r != Region)
		; Do nothing
		Return
	EndIf
	
	; Start with the first patrol
	Int i = -1

	; While there are still patrols to process
	While i < 127
		; Increment the patrol index
		i += 1

		; If the patrol exists, update settings
		If (Patrols[i] != None)
			Patrols[i].RegionUpdate = True
		EndIf
	EndWhile
EndFunction

;------------------------------------------------------------------------------\
; Update this settlement's mortality settings
;------------------------------------------------------------------------------/
Function Mortality(zYAWA_Manager_BOS m)
	; Start with the first patrol
	Int i = -1

	; While there are still patrols to process
	While i < 127
		; Increment the patrol index
		i += 1

		; If the patrol exists, update settings
		If (Patrols[i] != None)
			Patrols[i].UpdateMortality(m)
		EndIf
	EndWhile
EndFunction

;------------------------------------------------------------------------------\
; Update this settlement's actor settings
;------------------------------------------------------------------------------/
Function BaseActor(zYAWA_Manager_BOS m)
	; Start with the first patrol
	Int i = -1

	; While there are still patrols to process
	While i < 127
		; Increment the patrol index
		i += 1

		; If the patrol exists, update settings
		If (Patrols[i] != None)
			Patrols[i].UpdateBase = True
		EndIf
	EndWhile
EndFunction

;------------------------------------------------------------------------------\
; Check to see if this settlement's entrance should be moved
;------------------------------------------------------------------------------/
Int Function CheckDoor(zYAWA_Manager_BOS m, WorkshopScript w)
	; Start at the beginning of the list
	Int ci = 0

	; While there are unchecked objects in the list, continue checking
	While (ci < m.Doors.GetSize())
		; Get the current list object
		zYAWA_PatrolShift_BOS ps = m.Doors.GetAt(ci) As zYAWA_PatrolShift_BOS

		; If the object has this settlement's workshop or cell and marked undo
		If ((w == ps.Workshop || w.GetParentCell() == ps.ParentCell) && ps.Undo)
			; Reset this settlement's entrance and exit processing
			Entrance = Settlement.GetLinkedRef(m.Center)
			Return ci
		; If the object has this settlement's workshop
		ElseIf (w == ps.Workshop && ps.Workshop != None)
			; Move the entrace and exit processing
			Entrance = ps.NewPoint
			Return ci
		; If the object has this settlement's workshop cell
		ElseIf (w.GetParentCell() == ps.ParentCell && ps.ParentCell !=  None)
			; Move the entrance and exit processing
			Entrance = ps.NewPoint
			Return ci
		EndIf

		; Move to the next object
		ci += 1
	EndWhile

	; No move was found
	Return -1
EndFunction

;------------------------------------------------------------------------------\
; Check to see if this settlement's center should be moved
;------------------------------------------------------------------------------/
Int Function CheckCenter(zYAWA_Manager_BOS m, WorkshopScript w, Int id)
	; Start at the beginning of the list
	Int ci = 0

	; While there are unchecked objects in the list, continue checking
	While (ci < m.Centers.GetSize())
		; Get the current list object
		zYAWA_PatrolShift_BOS ps = m.Centers.GetAt(ci) As zYAWA_PatrolShift_BOS

		; If the object has this settlement's workshop or cell and marked undo
		If ((w == ps.Workshop || w.GetParentCell() == ps.ParentCell) && ps.Undo)
			; Reset this settlement's center and exit processing
			Center = Settlement.GetLinkedRef(m.Center)
			Return ci
		; If the object has this settlement's workshop
		ElseIf (w == ps.Workshop && ps.Workshop != None)
			; Move the center and exit processing
			Center = ps.NewPoint
			Return ci
		; If the object has this settlement's workshop cell
		ElseIf (w.GetParentCell() == ps.ParentCell && ps.ParentCell !=  None)
			; Move the center and exit processing
			Center = ps.NewPoint
			Return ci
		EndIf

		; Move to the next object
		ci += 1
	EndWhile

	; No move was found
	Return -1
EndFunction

;------------------------------------------------------------------------------\
; Update this settlement and its patrols
;------------------------------------------------------------------------------/
Function Update(zYAWA_Manager_BOS m, Int id)
	; The settlement defaults to unmoved
	Bool moved = False

	; Check to see if the settlement's door moved
	Int mi = CheckDoor(m, Settlement)

	; If the door moved
	If (mi >= 0)
		; Delete the move object and remove it from the list
		(m.Doors.GetAt(mi) As ObjectReference).Delete()
		m.Doors.RemoveAddedForm(m.Doors.GetAt(mi))

		; Mark this settlement as moved
		moved = True

		; Update this settlement's region
		Region = m.Regions[m.SortSettlement(Entrance)] As zYAWA_Region_BOS
	EndIf

	; Check to see if the settlement's center moved
	mi = CheckCenter(m, Settlement, id)

	; If the center moved
	If (mi >= 0)
		; Delete the move object and remove it from the list
		(m.Centers.GetAt(mi) As ObjectReference).Delete()
		m.Centers.RemoveAddedForm(m.Centers.GetAt(mi))

		; Mark this settlement as moved
		moved = True
	EndIf

	; Start with the first patrol
	Int i = -1

	; While there are still patrols to process
	While i < 127
		; Increment the patrol index
		i += 1

		; If the patrol exists, update it
		If (Patrols[i] != None)
			Patrols[i].Update(m, moved)
		EndIf
	EndWhile
EndFunction
